/**
 * Copyright (c) 2014 - 2019, Nordic Semiconductor ASA
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form, except as embedded into a Nordic
 *    Semiconductor ASA integrated circuit in a product or a software update for
 *    such product, must reproduce the above copyright notice, this list of
 *    conditions and the following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. Neither the name of Nordic Semiconductor ASA nor the names of its
 *    contributors may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 *
 * 4. This software, with or without modification, must only be used with a
 *    Nordic Semiconductor ASA integrated circuit.
 *
 * 5. Any software provided in binary form under this license must not be reverse
 *    engineered, decompiled, modified and/or disassembled.
 *
 * THIS SOFTWARE IS PROVIDED BY NORDIC SEMICONDUCTOR ASA "AS IS" AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL NORDIC SEMICONDUCTOR ASA OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */
/** @file
 *
 * @defgroup blinky_example_main main.c
 * @{
 * @ingroup blinky_example
 * @brief Blinky Example Application main file.
 *
 * This file contains the source code for a sample application to blink LEDs.
 *
 */

#include <stdbool.h>
#include <stdint.h>
#include "nrf_delay.h"
#include "boards.h"

#define CC_USED 0

void timer_setup(void)
{
    NRF_TIMER1->BITMODE = TIMER_BITMODE_BITMODE_32Bit;
    NRF_TIMER1->SHORTS = TIMER_SHORTS_COMPARE0_CLEAR_Enabled << TIMER_SHORTS_COMPARE0_CLEAR_Pos| TIMER_SHORTS_COMPARE0_STOP_Enabled << TIMER_SHORTS_COMPARE0_STOP_Pos;
    NRF_TIMER1->PRESCALER = 0;
}

/* Just a test function*/
static inline void sleep_us(uint32_t timeout_us)
{
    for (register uint32_t i = 0; i < timeout_us; i++)
    {
        __nop(); __nop(); __nop(); __nop(); 
        __nop(); __nop(); __nop(); __nop(); 
        __nop(); __nop(); __nop(); __nop(); 
        __nop(); __nop(); __nop(); __nop(); 
        __nop(); __nop(); __nop(); __nop(); 
        __nop(); __nop(); __nop(); __nop(); 
        __nop(); __nop(); __nop(); __nop(); 
        __nop(); __nop(); __nop(); __nop(); 
        __nop(); __nop(); __nop(); __nop(); 
        __nop(); __nop(); __nop(); __nop(); 
        __nop(); __nop(); __nop(); __nop(); 
        __nop(); __nop(); __nop(); __nop(); 
        __nop(); __nop(); __nop(); __nop(); 
        __nop(); __nop(); __nop(); __nop(); 
    }
}

#pragma push
#pragma O3
void timer_sleep(uint32_t timeout_us)
{

    if (timeout_us == 0) 
    {
        return;
    }
    if (timeout_us == 1)
    {
        /* Adjust the amount of NOPs here */
        __nop(); __nop(); __nop(); __nop(); 
        __nop(); __nop(); __nop(); __nop(); 
        __nop(); __nop(); __nop(); __nop(); 
        __nop(); __nop(); __nop(); __nop(); 
        __nop(); __nop(); __nop(); __nop(); 
        __nop(); __nop(); __nop(); __nop(); 
        __nop(); __nop(); __nop(); __nop(); 
        __nop(); __nop(); __nop(); __nop(); 
        __nop(); __nop(); __nop(); __nop(); 
        __nop(); __nop(); __nop(); __nop(); 
        __nop(); __nop(); __nop(); __nop(); 
        __nop(); __nop(); __nop(); __nop(); 
        __nop(); __nop();
        return;
    }

    NRF_TIMER1->CC[CC_USED] = (timeout_us << 4) - 18;
    NRF_TIMER1->TASKS_START = 1;
    while(NRF_TIMER1->EVENTS_COMPARE[CC_USED] == 0) 
    {
      /* Call __WFE() if loop is long */
    }
    NRF_TIMER1->EVENTS_COMPARE[CC_USED] = 0;
}
#pragma pop

/**
 * @brief Function for application main entry.
 */
int main(void)
{
    /* Configure board. */
    bsp_board_init(BSP_INIT_LEDS);
    NRF_CLOCK->TASKS_HFCLKSTART = 1;
    while (NRF_CLOCK->EVENTS_HFCLKSTARTED == 0);
    NRF_NVMC->ICACHECNF = NVMC_ICACHECNF_CACHEEN_Msk;
    timer_setup();
    /* Toggle LEDs. */
    while (true)
    {
        timer_sleep(2);
        NRF_P0->OUT ^= (1 << LED_1);

    }
}

/**
 *@}
 **/
